-- Hier finden sich zusätzliche Tabellen, die nicht durch den Formed generiert
-- werden.

CREATE TABLE ka_konfiguration_tbl
(
	id serial NOT NULL,
	bez varchar(64) NOT NULL,
	wert text,
	CONSTRAINT ka_konfiguration_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('fkz', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ka_name', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ee_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('fp_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ud_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('anon_transfer', 'on');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('max_speicherdauer', '90');

CREATE TABLE ka_dokumente_tbl
(
	id serial NOT NULL,
    	uuid_id CHAR(36) UNIQUE,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	groesse int4,
	mime varchar(64),
	daten bytea, 
	name varchar(128),
	CONSTRAINT ka_dokumente_tbl_pkey PRIMARY KEY (id)
);

-- ka_status_tbl
--
-- Speichert Zustand und letzte Zugriffszeit der Fallakte.
-- Mögliche Zustände:
--	Offen (1) 
--	Geschlossen (2)
--	Schwebend-Gelöscht (3)
--	Schwebend-Anonymisiert (4)
--	Anonymsiert (5)

CREATE TABLE ka_status_tbl
(
	id serial NOT NULL,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	status    int4,
	zugriff   timestamp,
	CONSTRAINT ka_status_tbl_pkey PRIMARY KEY (id)
);

-- ka_termine_tbl
-- 
-- Speichert grundlegende Angaben zu einem Termin. Weitergehende Angaben
-- sollten in einer extra Tabelle gespeichert werden.

CREATE TABLE ka_termine_tbl
(
	id serial NOT NULL,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	sdatum timestamp,
	edatum timestamp,
	name varchar(128),
	beschreibung text,
	art int4 DEFAULT 0, 
	CONSTRAINT ka_termine_tbl_pkey PRIMARY KEY (id)
);

-- ka_logbuch_art_tbl
--
-- Speichert die verschiedenen Arten von Logbuch-Eintraegen
--

CREATE TABLE ka_logbuch_art_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(80),
    CONSTRAINT ka_logbuch_art_tbl_pkey PRIMARY KEY (id)
);

GRANT SELECT ON ka_logbuch_art_tbl TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_art_tbl TO GROUP :admin_group;


INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (-2, 'Sonstige Angaben');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (-1, 'Keine Angabe');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 0, 'Automatischer Eintrag');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 1, 'Gespräch mit dem Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 2, 'Korrespondenz mit dem Jugendlichen (Telefonate/ Schriftverkehr)');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 3, 'Begleitung des Jugendlichen zu externen Stellen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 4, 'Sonstige Betreuung des Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 5, 'Schriftverkehr');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 6, 'Gespräch/Telefonat mit Kooperationspartnern');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 7, 'Gespräch/Telefonat mit Dritten');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 8, 'Förder- und Fallkonferenz');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 9, 'Erstgespräch'); -- Wird nicht mehr genutzt!
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (10, 'Kompetenzfeststellung');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (11, 'Gespräch mit Eltern/ Personensorgeberechtigten');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (12, 'Korrespondenz mit Eltern/ Personensorgeberechtigten (Telefonate/ Schriftverkehr)');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (13, 'Sonstige Maßnahmen der Elternarbeit');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (14, 'Fahrtwege');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (15, 'Dokumentation WASKA');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (16, 'Teambesprechung');

-- ka_logbuch_tbl
--
-- Verwaltet Logbucheintraege zu Fallakten
--

CREATE TABLE ka_logbuch_tbl (
    id         SERIAL NOT NULL,
    master_id  int4 REFERENCES master_tbl(id)         ON DELETE CASCADE,
    uuid_id    CHAR(36),
    datum      TIMESTAMP,
    bearbeiter int4 REFERENCES ka_benutzer_tbl(id)    ON DELETE SET DEFAULT,
    art        int4 REFERENCES ka_logbuch_art_tbl(id) ON DELETE SET DEFAULT,
    kurz_notiz VARCHAR(80),
    notiz      text,
    dauer      interval,
    UNIQUE (uuid_id),
    CONSTRAINT ka_logbuch_tbl_pkey PRIMARY KEY (id)
);

-- ka_nachrichten_tbl
--
-- Verwaltet Logbucheintraege zu Fallakten
--

CREATE TABLE ka_nachrichten_art_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(128),
    CONSTRAINT ka_nachrichten_art_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -2, 'Sonstiges');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -1, 'Keine Angabe');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES (  0, 'Systemnachricht');

CREATE TABLE ka_nachrichten_tbl (
    id         SERIAL NOT NULL,
    datum      TIMESTAMP,
    art        int4 REFERENCES ka_nachrichten_art_tbl(id) ON DELETE SET DEFAULT,
    titel      VARCHAR(128),
    nachricht  text,
    CONSTRAINT ka_nachrichten_tbl_pkey PRIMARY KEY (id)
);

CREATE TABLE nm_benutzer_nachrichten_tbl 
(
	id serial NOT NULL,
	benutzer_id int4 REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE,
	nachrichten_id int4 REFERENCES ka_nachrichten_tbl(id) ON DELETE CASCADE,
	gelesen timestamp DEFAULT now(),
	CONSTRAINT nm_benutzer_news_pkey PRIMARY KEY (id)
);

-- Tagging
CREATE TABLE tags (
    id          SERIAL PRIMARY KEY,
    label       VARCHAR(256) NOT NULL UNIQUE,
    description TEXT,
    systemtag   BOOLEAN DEFAULT TRUE
);

CREATE TABLE nm_tags_cases ( 
    id SERIAL PRIMARY KEY,
    master_id   integer NOT NULL REFERENCES master_tbl(id) ON DELETE CASCADE,
    tag_id    integer NOT NULL REFERENCES tags(id)       ON DELETE CASCADE,
    created   TIMESTAMP DEFAULT now(),
    UNIQUE (master_id, tag_id)
);

-- Add systemtags
INSERT INTO tags (id, label, description) VALUES (0, 'Alle','');
INSERT INTO tags (id, label, description) VALUES (1, 'konsistent','');
INSERT INTO tags (id, label, description) VALUES (2, 'phasenvollstaendig','');
INSERT INTO tags (id, label, description) VALUES (3, 'auswertbar','');
INSERT INTO tags (id, label, description) VALUES (4, 'adele-auswertbar','');

-- Move tags serial: userdefined tags start at 1000
ALTER SEQUENCE tags_id_seq RESTART 1000;

-- dbinfo
CREATE TABLE dbinfo (
    id INTEGER PRIMARY KEY NOT NULL,
    key   VARCHAR(128) NOT NULL,
    value VARCHAR(256) 
);

INSERT INTO dbinfo (id, key, value) VALUES (1, 'version', CURRENT_DATE);
INSERT INTO dbinfo (id, key, value) VALUES (2, 'datadump', NULL);

GRANT SELECT ON dbinfo TO PUBLIC;

-- temp tables
GRANT TEMP ON DATABASE :db_name TO PUBLIC;




