BEGIN;

SET ROLE :adm_ka_owner;

-- Log the update
INSERT INTO db_history (beschreibung, zeitpunkt) 
	VALUES ('update_1_0_2/logbuch/rc2-rc3.sql', current_timestamp);

UPDATE ka_logbuch_art_tbl SET beschreibung = 'Erstgespräch' WHERE id = 8;
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (10, 'Kompetenzfeststellung');

INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (100, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (101, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (102, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (103, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (104, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (105, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (106, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (107, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (108, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (109, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (110, 'dummy');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (111, 'dummy');

UPDATE ka_logbuch_tbl SET art = art + 102;

-- UPDATE ka_logbuch_art_tbl SET id = id + 1 WHERE id > -1 AND id < 10; -- Does not work!
CREATE FUNCTION shift_indices_logbuch_art() 
RETURNS void 
LANGUAGE 'plpgsql' 
AS $$
DECLARE
    mid integer;
BEGIN
    FOR mid IN SELECT id FROM ka_logbuch_art_tbl WHERE id > -1 AND id < 10 ORDER BY id DESC LOOP
        UPDATE ka_logbuch_art_tbl set id = id + 1 WHERE id = mid;
    END LOOP;
END;
$$;

SELECT shift_indices_logbuch_art();

DROP FUNCTION shift_indices_logbuch_art();

INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (0, 'Automatischer Eintrag');

SELECT * from ka_logbuch_art_tbl;

UPDATE ka_logbuch_tbl SET art = art - 102;

UPDATE ka_logbuch_tbl SET art = (art + 1) % 9 WHERE art > -1;

DELETE FROM ka_logbuch_art_tbl WHERE id >= 100 AND id <= 110;


-- UPDATE ka_logbuch_tbl SET art = art + 1 WHERE art > -1;
-- UPDATE ka_logbuch_tbl SET art = 0       WHERE art = 9;

ALTER TABLE ka_logbuch_tbl ALTER COLUMN art SET DEFAULT -1;

ALTER TABLE ka_logbuch_tbl ADD COLUMN kurz_notiz VARCHAR(80);

DROP VIEW ka_logbuch_tbl_view CASCADE;

CREATE OR REPLACE VIEW ka_logbuch_tbl_view 
    AS SELECT
        ka_logbuch_tbl.id         AS id,
        ka_logbuch_tbl.uuid_id    AS uuid_id,
        ka_logbuch_tbl.master_id  AS master_id,
        ka_logbuch_tbl.datum      AS datum,
        ka_logbuch_tbl.bearbeiter AS bearbeiter,
        ka_logbuch_tbl.kurz_notiz AS kurz_notiz,
        ka_logbuch_tbl.art        AS art,
        ka_logbuch_tbl.notiz      AS notiz,
        ka_logbuch_tbl.dauer      AS dauer
    FROM ka_logbuch_tbl
    INNER JOIN master_tbl ON
        ka_logbuch_tbl.master_id = master_tbl.id
    WHERE
        master_tbl.bearbeiter_id = getuserid()
        OR master_tbl.id IN (
            SELECT nm.master_id FROM nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id IN (
                SELECT nm2.gruppe_id FROM nm_benutzer_gruppe_tbl nm2 
                WHERE nm2.benutzer_id = getuserid())
        )
        OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_logbuch_update_rule
    AS ON UPDATE TO ka_logbuch_tbl_view 
	DO INSTEAD
        UPDATE ka_logbuch_tbl SET
            datum      = new.datum,
            bearbeiter = new.bearbeiter,
            art        = new.art,
            kurz_notiz = new.kurz_notiz,
            notiz      = new.notiz,
            dauer      = new.dauer
	    WHERE ka_logbuch_tbl.id = new.id;

CREATE OR REPLACE RULE ka_logbuch_delete_rule 
    AS ON DELETE TO  ka_logbuch_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_insert_rule
    AS ON INSERT TO ka_logbuch_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :admin_ka_group;


-- Damit Admins Logbucheintraege vornehmen koennen.
CREATE OR REPLACE FUNCTION create_logbuch_ds(mid integer, nuuid CHAR(36)) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
DECLARE
    lid integer;
BEGIN
    IF NOT (isEditorOrStandin(mid) OR pg_has_role('admin_ka','MEMBER')) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO ka_logbuch_tbl(master_id, datum, bearbeiter) VALUES (mid, now(), getuserid());

    SELECT INTO lid currval('ka_logbuch_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE ka_logbuch_tbl SET uuid_id = nuuid WHERE id = lid;
    ELSE
        PERFORM set_uuid('ka_logbuch_tbl', lid);
    END IF;

    RETURN lid;
END;
$$;

REVOKE ALL    ON FUNCTION create_logbuch_ds(integer, CHAR(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :admin_ka_group;


-- Trigger fuer das automatische Anlegen eines Logbucheintrages
-- beim Anlegen eines Masterdatensatzes.

CREATE OR REPLACE FUNCTION master_ds_created_logbook() RETURNS trigger AS $master_ds_created_logbook$
DECLARE
    lid integer;
BEGIN
    SELECT INTO lid create_logbuch_ds(NEW.id, NULL);
    UPDATE ka_logbuch_tbl_view
        SET art = 0, notiz = 'Fallakte angelegt.'
    WHERE
        id = lid;
    RETURN NEW;
END;
$master_ds_created_logbook$ LANGUAGE plpgsql;

--
-- Trigger, der nach Uebertragung der Fallakte an einen anderen
-- Bearbeiter den alten und neuen Bearbeiter eintraegt in das Logbuch eintraegt.
--
CREATE OR REPLACE FUNCTION master_ds_editor_changed() RETURNS trigger AS $master_ds_editor_changed$
DECLARE
    lid      integer;
    old_user RECORD;
    new_user RECORD;
BEGIN
    IF OLD.bearbeiter_id <> NEW.bearbeiter_id THEN

        SELECT INTO old_user * FROM ka_benutzer_tbl WHERE id = OLD.bearbeiter_id;
        SELECT INTO new_user * FROM ka_benutzer_tbl WHERE id = NEW.bearbeiter_id;

        SELECT INTO lid create_logbuch_ds(NEW.id, NULL);

        UPDATE ka_logbuch_tbl_view
            SET art = 0, 
            notiz = 'Fallakte übertragen von ' 
                    || coalesce(old_user.nachname, '-/-') || ', '
                    || coalesce(old_user.vorname,  '-/-') || ' an '
                    || coalesce(new_user.nachname, '-/-') || ', '
                    || coalesce(new_user.vorname,  '-/-')
        WHERE
            id = lid;
    END IF;
    RETURN NEW;
END;
$master_ds_editor_changed$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_editor_changed_trigger AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_editor_changed();


COMMIT;
