--
-- 07_default_value.sql
-- --------------------
-- Stellt eine Funktion get_default_value() zur Verfuegung, die zu einer
-- gegebenen Relation und Spalte den Default-Wert zurueck gibt.
--
BEGIN;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('update_1_0_4/07_default_value.sql', current_timestamp);


CREATE OR REPLACE FUNCTION get_default_value(relation_name text, attribute_name text)
RETURNS text 
LANGUAGE plpgsql STABLE AS
$$
DECLARE
    has_def bool;
    def     text;
BEGIN
    SELECT INTO has_def a.atthasdef FROM pg_class c 
    JOIN pg_attribute a ON c.oid = a.attrelid
    WHERE c.relname = relation_name AND a.attname = attribute_name;

    IF has_def IS NULL OR NOT has_def THEN
        RETURN NULL;
    END IF;

    SELECT INTO def pg_get_expr(i.adbin, i.oid) FROM 
    (SELECT c.oid, adnum, r.adbin
        FROM pg_class c 
        JOIN pg_attrdef r ON r.adrelid = c.oid
        WHERE c.relname = relation_name) i
        INNER JOIN pg_attribute p ON attrelid = i.oid AND i.adnum = p.attnum
        WHERE p.attname = attribute_name;
    RETURN def;
END;
$$;

REVOKE ALL ON FUNCTION get_default_value(text, text) FROM public;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :admin_ka_group;

COMMIT;
