BEGIN;
--
SET ROLE :db_adm;
INSERT INTO db_history (beschreibung, zeitpunkt)
        VALUES ('update_1_0_4/11_war_bereits_kunde.sql', current_timestamp);


DROP VIEW page_0_view;
DROP VIEW master_tbl_incm_view;
DROP VIEW master_tbl_eval_incm_view;
DROP VIEW master_tbl_eval_total_view;
DROP VIEW master_tbl_anonym_view;
DROP VIEW master_tbl_view;

ALTER TABLE master_tbl
    DROP CONSTRAINT master_tbl_war_bereits_kunde_fkey;

ALTER TABLE master_tbl
	DROP COLUMN war_bereits_kunde;

ALTER TABLE master_tbl
	ADD COLUMN war_cm_kunde INTEGER DEFAULT -1,
    ADD COLUMN war_sw_kunde INTEGER DEFAULT -1;

ALTER TABLE master_tbl
	ADD CONSTRAINT master_tbl_war_cm_kunde_fkey FOREIGN KEY (war_cm_kunde) REFERENCES einverstaendniserklaerung_tbl(id),
	ADD CONSTRAINT master_tbl_war_sw_kunde_fkey FOREIGN KEY (war_sw_kunde) REFERENCES einverstaendniserklaerung_tbl(id);

CREATE OR REPLACE VIEW page_0_view AS SELECT
    master_tbl.id AS id,
    master_tbl.fkz AS fkz,
    master_tbl.name_kompetenzagentur AS name_kompetenzagentur,
    master_tbl.vorname AS vorname,
    master_tbl.name AS name,
    master_tbl.erstgespraech AS erstgespraech,
    master_tbl.war_cm_kunde AS war_cm_kunde,
    master_tbl.war_sw_kunde AS war_sw_kunde,
    master_tbl.zugang_kompetenzagentur AS zugang_kompetenzagentur,
    master_tbl.kundennummer AS kundennummer,
    master_tbl.schulpflichtig AS schulpflichtig,
    master_tbl.noch_schueler AS noch_schueler,
    master_tbl.bei_eintritt_ist_der_jugendliche AS bei_eintritt_ist_der_jugendliche,
    master_tbl.rechtlicher_kontext AS rechtlicher_kontext,
    master_tbl.schnittstelle_zu AS schnittstelle_zu
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_0_update_rule AS ON UPDATE TO page_0_view
DO INSTEAD UPDATE master_tbl SET
    fkz = NEW.fkz,
    name_kompetenzagentur = NEW.name_kompetenzagentur,
    vorname = NEW.vorname,
    name = NEW.name,
    erstgespraech = NEW.erstgespraech,
    war_cm_kunde = NEW.war_cm_kunde,
    war_sw_kunde = NEW.war_sw_kunde,
    zugang_kompetenzagentur = NEW.zugang_kompetenzagentur,
    kundennummer = NEW.kundennummer,
    schulpflichtig = NEW.schulpflichtig,
    noch_schueler = NEW.noch_schueler,
    bei_eintritt_ist_der_jugendliche = NEW.bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext = NEW.rechtlicher_kontext,
    schnittstelle_zu = NEW.schnittstelle_zu
WHERE id = NEW.id;

GRANT SELECT ON page_0_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_0_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_0_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_0_delete_rule AS ON DELETE TO page_0_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_0_insert_rule AS ON INSERT TO page_0_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW master_tbl_view
AS SELECT master_tbl.* FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE master_tbl_update_rule AS ON UPDATE TO master_tbl_view
DO INSTEAD UPDATE master_tbl SET
    einverstaendniserklaerung = NEW.einverstaendniserklaerung,
    phase = NEW.phase,
    fkz = NEW.fkz,
    name_kompetenzagentur = NEW.name_kompetenzagentur,
    vorname = NEW.vorname,
    name = NEW.name,
    erstgespraech = NEW.erstgespraech,
    war_cm_kunde = NEW.war_cm_kunde,
    war_sw_kunde = NEW.war_sw_kunde,
    zugang_kompetenzagentur = NEW.zugang_kompetenzagentur,
    kundennummer = NEW.kundennummer,
    schulpflichtig = NEW.schulpflichtig,
    noch_schueler = NEW.noch_schueler,
    bei_eintritt_ist_der_jugendliche = NEW.bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext = NEW.rechtlicher_kontext,
    schnittstelle_zu = NEW.schnittstelle_zu,
    kontaktdaten_arge = NEW.kontaktdaten_arge,
    kundennummer_arge = NEW.kundennummer_arge,
    kontaktdaten_agentur_fuer_arbeit = NEW.kontaktdaten_agentur_fuer_arbeit,
    kundennummer_agentur_fuer_arbeit = NEW.kundennummer_agentur_fuer_arbeit,
    kundennummer_jugendamt = NEW.kundennummer_jugendamt,
    kontaktdaten_jugendamt = NEW.kontaktdaten_jugendamt,
    kontaktdaten_schule = NEW.kontaktdaten_schule,
    schulname = NEW.schulname,
    kontaktdaten_gerichtshilfe = NEW.kontaktdaten_gerichtshilfe,
    kundennummer_gerichtshilfe = NEW.kundennummer_gerichtshilfe,
    kontaktdaten_weitere_institutionen = NEW.kontaktdaten_weitere_institutionen,
    vorname_1 = NEW.vorname_1,
    name_1 = NEW.name_1,
    geschlecht = NEW.geschlecht,
    geburtsdatum = NEW.geburtsdatum,
    familienstand = NEW.familienstand,
    strasse = NEW.strasse,
    plz = NEW.plz,
    wohnort = NEW.wohnort,
    groesse_wohnort = NEW.groesse_wohnort,
    vorwahl = NEW.vorwahl,
    telefon_1 = NEW.telefon_1,
    mobiltelefon = NEW.mobiltelefon,
    email = NEW.email,
    telefon_sonstige = NEW.telefon_sonstige,
    zweiter_wohnort = NEW.zweiter_wohnort,
    geburtsland = NEW.geburtsland,
    geburtsland_sonstige_nennung = NEW.geburtsland_sonstige_nennung,
    muttersprache = NEW.muttersprache,
    muttersprache_sonstige_nennung = NEW.muttersprache_sonstige_nennung,
    staatsangehoerigkeit = NEW.staatsangehoerigkeit,
    staatsangehoerigkeit_sonstige_nennung = NEW.staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus = NEW.aufenthaltsstatus,
    aufenthaltsdauer = NEW.aufenthaltsdauer,
    vater = NEW.vater,
    mutter = NEW.mutter,
    familiaere_situation_1 = NEW.familiaere_situation_1,
    vorname_2 = NEW.vorname_2,
    name_2 = NEW.name_2,
    vorwahl_1 = NEW.vorwahl_1,
    telefon_1_1 = NEW.telefon_1_1,
    mobiltelefon_1 = NEW.mobiltelefon_1,
    email_1 = NEW.email_1,
    geschwister = NEW.geschwister,
    anzahl_kinder = NEW.anzahl_kinder,
    haushalt_jugendlicher = NEW.haushalt_jugendlicher,
    alleinerziehend = NEW.alleinerziehend,
    familie_selbststaendig = NEW.familie_selbststaendig,
    familie_staatliche_unterstuetzung = NEW.familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig = NEW.jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung = NEW.jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld = NEW.jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt = NEW.wohnaufenthalt,
    arzt_gutachten_physisch = NEW.arzt_gutachten_physisch,
    physische_beeintraechtigungen = NEW.physische_beeintraechtigungen,
    physische_beeintraechtigungen_text = NEW.physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch = NEW.arzt_gutachten_psychisch,
    psychische_beeintraechtigungen = NEW.psychische_beeintraechtigungen,
    psychische_beeintraechtigungen_text = NEW.psychische_beeintraechtigungen_text,
    anerkannte_behinderung = NEW.anerkannte_behinderung,
    mutter_2 = NEW.mutter_2,
    vater_2 = NEW.vater_2,
    grosseltern = NEW.grosseltern,
    geschwister_2 = NEW.geschwister_2,
    pflegefamilie_1 = NEW.pflegefamilie_1,
    stiefmutter = NEW.stiefmutter,
    stiefvater = NEW.stiefvater,
    verwandte_1 = NEW.verwandte_1,
    partner = NEW.partner,
    heimbetreuerin = NEW.heimbetreuerin,
    betreuerin = NEW.betreuerin,
    lehrerin = NEW.lehrerin,
    mitbewohner = NEW.mitbewohner,
    partner_mutter = NEW.partner_mutter,
    partner_vater = NEW.partner_vater,
    freund = NEW.freund,
    keine_bezugsperson = NEW.keine_bezugsperson,
    hobbies_1 = NEW.hobbies_1,
    hobbies_text = NEW.hobbies_text,
    mitarbeit_organisationen = NEW.mitarbeit_organisationen,
    organisationen_vereine_text = NEW.organisationen_vereine_text,
    sonstiges_1 = NEW.sonstiges_1,
    sonstiges_text = NEW.sonstiges_text,
    clique = NEW.clique,
    freundin = NEW.freundin,
    parterin = NEW.parterin,
    schule_2 = NEW.schule_2,
    klassenstufe = NEW.klassenstufe,
    klassenwiederholung = NEW.klassenwiederholung,
    abgang = NEW.abgang,
    nicht_anwesende_tage = NEW.nicht_anwesende_tage,
    unentschuldigte_tage = NEW.unentschuldigte_tage,
    schulabschluss_1 = NEW.schulabschluss_1,
    schulabschlusserwerbsland = NEW.schulabschlusserwerbsland,
    schulabschlusserwerbsland_sonstige_nennung = NEW.schulabschlusserwerbsland_sonstige_nennung,
    abschluss_betriebliche_ausbildung = NEW.abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf = NEW.abschluss_im_ausbildungsberuf,
    ausbildungsdauer = NEW.ausbildungsdauer,
    wann_abgeschlossen = NEW.wann_abgeschlossen,
    weitere_ausbildungen = NEW.weitere_ausbildungen,
    weitere_ausbildungen_text = NEW.weitere_ausbildungen_text,
    dauer_weitere_ausbildungen = NEW.dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen = NEW.abgebrochene_ausbildungen,
    abgebrochene_ausbildungen_text = NEW.abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung = NEW.abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1 = NEW.abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1 = NEW.ausbildungsdauer_1,
    wann_abgeschlossen_1 = NEW.wann_abgeschlossen_1,
    weitere_ausbildungen_1 = NEW.weitere_ausbildungen_1,
    weitere_ausbildungen_text_1 = NEW.weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1 = NEW.dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1 = NEW.abgebrochene_ausbildungen_1,
    abgebrochene_ausbildungen_text_1 = NEW.abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme = NEW.massnahmeteilnahme,
    letzte_massnahme = NEW.letzte_massnahme,
    finanzierung = NEW.finanzierung,
    austrittsdatum = NEW.austrittsdatum,
    dauer = NEW.dauer,
    weitere_massnahmen = NEW.weitere_massnahmen,
    massnahme_1 = NEW.massnahme_1,
    massnahme1_finanzierung = NEW.massnahme1_finanzierung,
    massnahme_2 = NEW.massnahme_2,
    massnahme2_finanzierung = NEW.massnahme2_finanzierung,
    massnahme_3 = NEW.massnahme_3,
    massnahme3_finanzierung = NEW.massnahme3_finanzierung,
    abgebrochene_massnahmen = NEW.abgebrochene_massnahmen,
    erwerbstaetigkeit_job = NEW.erwerbstaetigkeit_job,
    letzte_taetigkeit = NEW.letzte_taetigkeit,
    dauer_1 = NEW.dauer_1,
    austrittsdatum_1 = NEW.austrittsdatum_1,
    weitere_taetigkeiten = NEW.weitere_taetigkeiten,
    weitere_jobs_text = NEW.weitere_jobs_text,
    dauer_weitere_taetigkeiten = NEW.dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1 = NEW.arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit = NEW.anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung = NEW.dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt = NEW.dauer_insgesamt,
    bezug_alg1 = NEW.bezug_alg1,
    bezug_alg2 = NEW.bezug_alg2,
    wehrpflicht_zivildienst_freiwilligenjahr_1 = NEW.wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes = NEW.art_des_dienstes,
    dauer_2 = NEW.dauer_2,
    mutterschutz_erziehungsurlaub = NEW.mutterschutz_erziehungsurlaub,
    anzahl = NEW.anzahl,
    dauer_letzter_erziehungsurlaub = NEW.dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1 = NEW.dauer_insgesamt_1,
    kompetenzfeststellung = NEW.kompetenzfeststellung,
    datum_feststellung = NEW.datum_feststellung,
    zielsetzung = NEW.zielsetzung,
    begleitung = NEW.begleitung,
    welche_vermittlung_nennung = NEW.welche_vermittlung_nennung,
    beratungsfortsetzung = NEW.beratungsfortsetzung,
    datum_ende_beratung = NEW.datum_ende_beratung,
    freie_dokumentation = NEW.freie_dokumentation,
    vereinbarung_foerderplan = NEW.vereinbarung_foerderplan,
    bildender_bereich = NEW.bildender_bereich,
    bereich_berufliche_qualifizierung = NEW.bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung = NEW.bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung = NEW.bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf = NEW.sonstiger_foerderbedarf,
    sonstiger_foerderbedarf_nennung = NEW.sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start = NEW.foerderzeitraum_start,
    foerderzeitraum_ende = NEW.foerderzeitraum_ende,
    dokumentation_zum_foerderplan = NEW.dokumentation_zum_foerderplan,
    anzahl_kontakte_termine = NEW.anzahl_kontakte_termine,
    anzahl_der_stunden = NEW.anzahl_der_stunden,
    zeitraum_fallmanagement = NEW.zeitraum_fallmanagement,
    beendigung_5 = NEW.beendigung_5,
    art_beendigung = NEW.art_beendigung,
    abbruch_durch = NEW.abbruch_durch,
    gruende_abbruch = NEW.gruende_abbruch,
    gruende_sonstiges_nennung_4 = NEW.gruende_sonstiges_nennung_4,
    vermittlung = NEW.vermittlung,
    vermittlung_nennung = NEW.vermittlung_nennung,
    veraenderungen_rechtskreis = NEW.veraenderungen_rechtskreis,
    allgemein_bildender_schulabschluss = NEW.allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung = NEW.allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss = NEW.massnahmeabschluss,
    schulart = NEW.schulart,
    schule_sonstiges_nennung = NEW.schule_sonstiges_nennung,
    klassenstufe_1 = NEW.klassenstufe_1,
    ausbildung = NEW.ausbildung,
    ausbildung_sonstiges_nennung = NEW.ausbildung_sonstiges_nennung,
    umschulung = NEW.umschulung,
    umschulung_nennung = NEW.umschulung_nennung,
    arbeitstaetigkeit = NEW.arbeitstaetigkeit,
    erwerbstatus = NEW.erwerbstatus,
    erwerbsstatus_sonstiges_nennung = NEW.erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang = NEW.beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung = NEW.kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung = NEW.kategorien_sonstiges_nennung,
    beendigung_nachbetreuung = NEW.beendigung_nachbetreuung,
    zeitraum_nachbetreuung = NEW.zeitraum_nachbetreuung,
    verbleib_nachbetreuung = NEW.verbleib_nachbetreuung,
    mitgliedschaft_organisationen_vereinen = NEW.mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft = NEW.intensitaet_mitgliedschaft,
    nutzung_kulturangeboten = NEW.nutzung_kulturangeboten,
    haeufigkeit = NEW.haeufigkeit,
    soziales_engagement = NEW.soziales_engagement,
    unternehmungen_freunden = NEW.unternehmungen_freunden,
    unternehmungen_familie = NEW.unternehmungen_familie,
    peergroup = NEW.peergroup,
    familie_1 = NEW.familie_1,
    beziehung = NEW.beziehung,
    arbeit_schule = NEW.arbeit_schule,
    bester_freund_freundin = NEW.bester_freund_freundin,
    einhaltungtermine = NEW.einhaltungtermine,
    eigenstaendigeamtsgaenge = NEW.eigenstaendigeamtsgaenge,
    entwurflebensplaene = NEW.entwurflebensplaene,
    umgangmitproblemen = NEW.umgangmitproblemen,
    einschaetzung_zielerreichung = NEW.einschaetzung_zielerreichung,
    erreichte_qualifikation = NEW.erreichte_qualifikation,
    integration_ist_nachhaltig = NEW.integration_ist_nachhaltig,
    selbststaendige_berufswegplanung = NEW.selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung = NEW.eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme = NEW.soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben = NEW.teilnahme_gesellschaftliches_leben,
    freie_dokumentation_abschlussbewertung = NEW.freie_dokumentation_abschlussbewertung
WHERE id = NEW.id;

CREATE OR REPLACE RULE master_tbl_delete_rule AS ON DELETE TO master_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_insert_rule AS ON INSERT TO master_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON master_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON master_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW master_tbl_anonym_view
AS SELECT
    id,
    uuid_id,
    bearbeiter_id,
    einverstaendniserklaerung,
    phase,
    NULL::VARCHAR(60) AS fkz,
    NULL::VARCHAR(60) AS name_kompetenzagentur,
    NULL::VARCHAR(60) AS vorname,
    hash_value(name, vorname) AS name,
    erstgespraech,
    war_cm_kunde,
    war_sw_kunde,
    zugang_kompetenzagentur,
    NULL::VARCHAR(100) AS kundennummer,
    schulpflichtig,
    noch_schueler,
    bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext,
    schnittstelle_zu,
    NULL::VARCHAR(100) AS kontaktdaten_arge,
    NULL::VARCHAR(100) AS kundennummer_arge,
    NULL::VARCHAR(100) AS kontaktdaten_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_schule,
    NULL::VARCHAR(100) AS schulname,
    NULL::VARCHAR(100) AS kontaktdaten_gerichtshilfe,
    NULL::VARCHAR(30) AS kundennummer_gerichtshilfe,
    NULL::TEXT AS kontaktdaten_weitere_institutionen,
    NULL::VARCHAR(60) AS vorname_1,
    NULL::VARCHAR(60) AS name_1,
    geschlecht,
    get_age(geburtsdatum) AS geburtsdatum,
    familienstand,
    NULL::VARCHAR(200) AS strasse,
    shorten_value(plz, 3) AS plz,
    NULL::VARCHAR(60) AS wohnort,
    groesse_wohnort,
    NULL::VARCHAR(60) AS vorwahl,
    NULL::VARCHAR(60) AS telefon_1,
    NULL::VARCHAR(120) AS mobiltelefon,
    NULL::VARCHAR(120) AS email,
    NULL::TEXT AS telefon_sonstige,
    NULL::VARCHAR(120) AS zweiter_wohnort,
    geburtsland,
    NULL::VARCHAR(60) AS geburtsland_sonstige_nennung,
    muttersprache,
    NULL::VARCHAR(60) AS muttersprache_sonstige_nennung,
    staatsangehoerigkeit,
    NULL::VARCHAR(60) AS staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus,
    aufenthaltsdauer,
    vater,
    mutter,
    familiaere_situation_1,
    NULL::VARCHAR(60) AS vorname_2,
    NULL::VARCHAR(60) AS name_2,
    NULL::VARCHAR(60) AS vorwahl_1,
    NULL::VARCHAR(60) AS telefon_1_1,
    NULL::VARCHAR(60) AS mobiltelefon_1,
    NULL::VARCHAR(120) AS email_1,
    geschwister,
    anzahl_kinder,
    haushalt_jugendlicher,
    alleinerziehend,
    familie_selbststaendig,
    familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt,
    arzt_gutachten_physisch,
    check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch) AS physische_beeintraechtigungen,
    check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch) AS physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch,
    check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen,
    check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen_text,
    anerkannte_behinderung,
    mutter_2,
    vater_2,
    grosseltern,
    geschwister_2,
    pflegefamilie_1,
    stiefmutter,
    stiefvater,
    verwandte_1,
    partner,
    heimbetreuerin,
    betreuerin,
    lehrerin,
    mitbewohner,
    partner_mutter,
    partner_vater,
    freund,
    keine_bezugsperson,
    hobbies_1,
    hobbies_text,
    mitarbeit_organisationen,
    organisationen_vereine_text,
    sonstiges_1,
    sonstiges_text,
    clique,
    freundin,
    parterin,
    schule_2,
    klassenstufe,
    klassenwiederholung,
    abgang,
    nicht_anwesende_tage,
    unentschuldigte_tage,
    schulabschluss_1,
    schulabschlusserwerbsland,
    schulabschlusserwerbsland_sonstige_nennung,
    abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf,
    ausbildungsdauer,
    wann_abgeschlossen,
    weitere_ausbildungen,
    weitere_ausbildungen_text,
    dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen,
    abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1,
    wann_abgeschlossen_1,
    weitere_ausbildungen_1,
    weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1,
    abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme,
    letzte_massnahme,
    finanzierung,
    austrittsdatum,
    dauer,
    weitere_massnahmen,
    massnahme_1,
    massnahme1_finanzierung,
    massnahme_2,
    massnahme2_finanzierung,
    massnahme_3,
    massnahme3_finanzierung,
    abgebrochene_massnahmen,
    erwerbstaetigkeit_job,
    letzte_taetigkeit,
    dauer_1,
    austrittsdatum_1,
    weitere_taetigkeiten,
    weitere_jobs_text,
    dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt,
    bezug_alg1,
    bezug_alg2,
    wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes,
    dauer_2,
    mutterschutz_erziehungsurlaub,
    anzahl,
    dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1,
    kompetenzfeststellung,
    datum_feststellung,
    zielsetzung,
    begleitung,
    welche_vermittlung_nennung,
    beratungsfortsetzung,
    datum_ende_beratung,
    NULL::TEXT AS freie_dokumentation,
    vereinbarung_foerderplan,
    bildender_bereich,
    bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf,
    sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start,
    foerderzeitraum_ende,
    NULL::TEXT AS dokumentation_zum_foerderplan,
    anzahl_kontakte_termine,
    anzahl_der_stunden,
    zeitraum_fallmanagement,
    beendigung_5,
    art_beendigung,
    abbruch_durch,
    gruende_abbruch,
    gruende_sonstiges_nennung_4,
    vermittlung,
    vermittlung_nennung,
    veraenderungen_rechtskreis,
    allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss,
    schulart,
    schule_sonstiges_nennung,
    klassenstufe_1,
    ausbildung,
    ausbildung_sonstiges_nennung,
    umschulung,
    umschulung_nennung,
    arbeitstaetigkeit,
    erwerbstatus,
    erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung,
    beendigung_nachbetreuung,
    zeitraum_nachbetreuung,
    verbleib_nachbetreuung,
    mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft,
    nutzung_kulturangeboten,
    haeufigkeit,
    soziales_engagement,
    unternehmungen_freunden,
    unternehmungen_familie,
    peergroup,
    familie_1,
    beziehung,
    arbeit_schule,
    bester_freund_freundin,
    einhaltungtermine,
    eigenstaendigeamtsgaenge,
    entwurflebensplaene,
    umgangmitproblemen,
    einschaetzung_zielerreichung,
    erreichte_qualifikation,
    integration_ist_nachhaltig,
    selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben,
    NULL::TEXT AS freie_dokumentation_abschlussbewertung FROM master_tbl_view;

CREATE OR REPLACE RULE master_tbl_anonym_delete_rule AS ON DELETE TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_anonym_insert_rule AS ON INSERT TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON master_tbl_anonym_view TO GROUP :admin_ka_group;

-- Views

CREATE OR REPLACE  VIEW master_tbl_eval_total_view AS
    SELECT m.* FROM master_tbl m JOIN ka_status_tbl s
        ON m.id = s.master_id
        WHERE
            s.status <> 3 AND
            (m.bearbeiter_id = (getuserid())::integer)
            OR pg_has_role('admin_ka','MEMBER');

GRANT SELECT ON master_tbl_eval_total_view TO :cm_ka_group;
GRANT SELECT ON master_tbl_eval_total_view TO :admin_ka_group;


CREATE OR REPLACE  VIEW master_tbl_eval_incm_view AS
    SELECT * FROM master_tbl_eval_total_view
        WHERE
            bereich_berufliche_qualifizierung = 1 OR
            bereich_berufsvorbereitung = 1 OR
            bildender_bereich = 1 OR
            bereich_lebensbewaeltigung = 1 OR
            sonstiger_foerderbedarf = 1;

GRANT SELECT ON public.master_tbl_eval_incm_view TO :cm_ka_group;
GRANT SELECT ON public.master_tbl_eval_incm_view TO :admin_ka_group;

CREATE OR REPLACE  VIEW master_tbl_incm_view AS
    SELECT * FROM master_tbl_view
        WHERE
            bereich_berufliche_qualifizierung = 1 OR
            bereich_berufsvorbereitung = 1 OR
            bildender_bereich = 1 OR
            bereich_lebensbewaeltigung = 1 OR
            sonstiger_foerderbedarf = 1;

GRANT SELECT ON public.master_tbl_incm_view TO :cm_ka_group;
GRANT SELECT ON public.master_tbl_incm_view TO :admin_ka_group;

COMMIT;
