BEGIN;
-- ka_status_tbl
--
-- Speichert Zustand und letzte Zugriffszeit der Fallakte.
-- Mögliche Zustände:
--	Offen (1) 
--	Geschlossen (2)
--	Schwebend-Gelöscht (3)
--	Schwebend-Anonymisiert (4)
--	Anonymsiert (5)

CREATE TABLE ka_status_tbl
(
	id serial NOT NULL,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	status int4,
	zugriff timestamp,
	CONSTRAINT ka_status_tbl_pkey PRIMARY KEY (id)
);

ALTER TABLE ka_status_tbl OWNER TO :db_adm;

-- VIEW  for ka_status_tbl

CREATE OR REPLACE VIEW ka_status_tbl_view AS 
SELECT ka_status_tbl.id, ka_status_tbl.master_id,
ka_status_tbl.status, ka_status_tbl.zugriff FROM ka_status_tbl;

CREATE OR REPLACE RULE ka_status_delete_rule AS ON DELETE TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_insert_rule AS ON INSERT TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_update_rule AS ON UPDATE TO ka_status_tbl_view 
	DO INSTEAD UPDATE ka_status_tbl SET
	status = new.status,
	zugriff = new.zugriff
	WHERE id = new.id;

REVOKE ALL ON TABLE ka_status_tbl_view FROM public;
GRANT SELECT ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_status_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :admin_ka_group;
ALTER TABLE ka_status_tbl_view OWNER TO :db_adm;

-- Funktionen zum Anlegen neuer (Teile von) Fallakten
-- Diese sind nur für die Gruppe CM_KA aufrufbar (Admin_KA darf keine neuen 
-- Akten anlegen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION create_master_ds() RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    standin integer;
BEGIN
INSERT INTO master_tbl (bearbeiter_id) VALUES (getuserid());
SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = getuserid();
IF standin IS NOT NULL THEN 
    INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id) VALUES (standin, 
currval('master_tbl_id_seq'));
END IF;
PERFORM create_status_ds(CAST (currval('master_tbl_id_seq') AS integer));
RETURN currval('master_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_master_ds() FROM public;
GRANT EXECUTE ON FUNCTION create_master_ds() TO GROUP :cm_ka_group;
ALTER FUNCTION create_master_ds() OWNER TO :db_adm;
--
-- create_status_ds(mid integer)
-- legt einen statusdatensatz für eine Fallakte an
-- 
-- deletion of the entry is done by ON DELETE CONSTRAINT

CREATE OR REPLACE FUNCTION create_status_ds(mid integer)
RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER 
    AS $$
BEGIN
INSERT INTO ka_status_tbl (master_id, zugriff, status) VALUES (mid, now(), 1);
END;
$$;

REVOKE ALL ON FUNCTION create_status_ds( integer ) FROM public;
GRANT EXECUTE ON FUNCTION create_status_ds(integer) TO GROUP :cm_ka_group;
ALTER FUNCTION create_status_ds( integer ) OWNER TO :db_adm;

-- Adding entrie for max savetime in ka_agency
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('max_speicherdauer', '0');
-- Adding status entries for cases which do not have an status already
CREATE OR REPLACE FUNCTION create_status_entries() RETURNS integer AS '
DECLARE
    case RECORD;
BEGIN
    FOR case IN SELECT id, einverstaendniserklaerung FROM master_tbl WHERE id not in (select master_id
    from ka_status_tbl) LOOP
	PERFORM create_status_ds(case.id);
	IF case.einverstaendniserklaerung = 0 THEN
		UPDATE ka_status_tbl_view SET status = 3 WHERE master_id = case.id;
	END IF;
    END LOOP;
    RETURN 1;
END;
' LANGUAGE plpgsql;
SELECT create_status_entries();
DROP function create_status_entries();

COMMIT;
