\connect postgres postgres
BEGIN;
UPDATE pg_database SET datallowconn = TRUE where datname='waska_template';
COMMIT;

\connect waska_maintenance postgres
BEGIN;
-- waska_init_database( agentur ) returns text
-- Initialisieren einer Datenbank, mit entsprechenden Rollen für die KA und
-- Login-Rolle 'ka_<agentur>_adm'. Rückgabewert ist das initiale Passwort der
-- Login-Rolle 'ka_<NAME AGENTUR>_adm'. Diese Funktion steht der Rolle "PT-DLR"
-- zur Verfügung.
-- 
-- Die Datenbank muss vorher angelegt sein, entsprechend muss auch die
-- Prüfung auf korrekten Namen vorher stattfinden. (CREATE DATABASE kann
-- nicht in einem Skript aufgerufen werden)
CREATE OR REPLACE FUNCTION waska_init_database (
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE
    db_name   varchar(64);
    adm_group varchar(64);
    adm_user  varchar(64);
    cm_group  varchar(64);
    pb_group  varchar(64);
    pw        varchar(13);
  BEGIN
    -- Gruppen fuer die KA
    adm_group := quote_ident('ka_' || agency || '_admin_ka');
    cm_group :=  quote_ident('ka_' || agency || '_cm_ka');
    pb_group :=  quote_ident('ka_' || agency || '_pb_ka');
    EXECUTE 'CREATE ROLE ' || adm_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'GRANT admin_ka TO ' || adm_group;
    EXECUTE 'CREATE ROLE ' || cm_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'CREATE ROLE ' || pb_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'GRANT ' || pb_group || ' TO ' || adm_group;

    -- Administratives Konto (Besitzer der DB) einrichten
    db_name := quote_ident('ka_' || agency || '_db');
    adm_user := quote_ident('ka_' || agency || '_adm');
    pw := waska_passgen(12);
    EXECUTE 'CREATE ROLE ' || adm_user
        || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
        || ' PASSWORD ' || quote_literal(pw);
    EXECUTE 'REVOKE ALL ON DATABASE ' || db_name || ' FROM PUBLIC';
    EXECUTE 'ALTER DATABASE ' || db_name
            || ' OWNER TO ' || adm_user;
    EXECUTE 'GRANT ' || adm_group || ' TO ' || adm_user;

    RETURN pw::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_init_database( varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_init_database( varchar ) TO pt_dlr;

COMMIT;
\connect waska_template postgres
BEGIN;
-- waska_create_role( nutzer, gruppenrolle, agentur ) returns text 
-- Anlegen einer Login-Rolle 'ka_<agentur>_<nutzer>' in der gruppenrolle:
-- 'CM-KA' oder 'Admin-KA'. Es wird anhand der Namensräume überprüft, ob die
-- aufrufende Rolle berechtigt ist, für die angegebene KA Nutzer anzulegen.
-- Rückgabewert ist die Loginrolle und das initiale Passwort.
CREATE OR REPLACE FUNCTION waska_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pw       varchar(9);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin-ka' THEN
        db_group := quote_ident('ka_' || agency || '_admin_ka');
      ELSIF lower(grouprole) = 'pb-ka' THEN
        db_group := quote_ident('ka_' || agency || '_pb_ka');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm_ka');
      END IF;
      pw := waska_passgen(8);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(9);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    TO admin_ka;
COMMIT;

\connect postgres postgres
BEGIN;
UPDATE pg_database SET datallowconn = FALSE where datname='waska_template';
COMMIT;
