-- 
-- get_master_id_from_uuid(myrel varchar, uuid char) 
-- Author: Torsten Irlaender 
--
-- Gibt die master id des datensatzes mit der übergebenen uuid in der
-- angegebenen realtion wieder. Wenn ein solcher datensatz nicht existiert, wird
-- NULL zurueck gegeben.
--
BEGIN;
SET ROLE :db_adm;
CREATE OR REPLACE FUNCTION get_masterid_from_uuid(myrel varchar(128), uuid char(36))
    RETURNS integer AS
$$
DECLARE
    myid integer;
BEGIN
    IF myrel = 'master' THEN
        SELECT INTO myid id from master_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'angebote_berufsvorbereitung' THEN
        SELECT INTO myid master_id from rg_angebote_berufsvorbereitung_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'angebote_bildenden_bereich' THEN
        SELECT INTO myid master_id from rg_angebote_bildenden_bereich_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'angebote_berufliche_qualifizierung' THEN
        SELECT INTO myid master_id from rg_angebote_berufliche_qualifizierung_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'angebote_lebensbewaeltigung' THEN
        SELECT INTO myid master_id from rg_angebote_lebensbewaeltigung_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'kompetenzfestellung' THEN
        SELECT INTO myid master_id from rg_kompetenzfestellung_tbl WHERE uuid_id = uuid;
    END IF;
    RETURN myid;
END;
$$ LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER;
REVOKE ALL ON FUNCTION get_masterid_from_uuid(varchar, char) FROM public;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :admin_ka_group;
COMMIT;
