BEGIN;
SET ROLE :db_adm;  
--
-- Automatisches Setzen eines Zeitstempels in der ka_status_tbl
-- Diese Trigger sorgen dafür, dass bei jeder Bearbeitung der Fallakten der
-- Zeitpunkt der Bearbeitung gespeichert wird.
--
CREATE OR REPLACE FUNCTION set_lastaccess_time( masterid integer ) 
RETURNS void AS 
$$
BEGIN
    UPDATE ka_status_tbl_view set zugriff = now() WHERE master_id = masterid;  
END;
$$ LANGUAGE plpgsql VOLATILE;

CREATE OR REPLACE FUNCTION master_lastaccess_log() RETURNS trigger AS $master_lastaccess_log$
BEGIN
    EXECUTE set_lastaccess_time(NEW.id);
    RETURN NEW;
END;
$master_lastaccess_log$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION status_lastaccess_log() RETURNS trigger AS $status_lastaccess_log$
BEGIN
IF NEW.status <> OLD.status AND OLD.zugriff IS NOT NULL THEN
    EXECUTE set_lastaccess_time(NEW.master_id);
END IF;
RETURN NEW;
END;
$status_lastaccess_log$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_lastaccess_log() RETURNS trigger AS $rg_lastaccess_log$
BEGIN
IF TG_OP = 'DELETE' THEN
    EXECUTE set_lastaccess_time(OLD.master_id);
    RETURN OLD;
ELSE
    EXECUTE set_lastaccess_time(NEW.master_id);
    RETURN NEW;
END IF;
END;
$rg_lastaccess_log$ LANGUAGE plpgsql;

CREATE TRIGGER t1_master_tbl_update AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_lastaccess_log();

-- rg_kompetenzfestellung_tbl
CREATE TRIGGER t1_rg_kompetenzfestellung_tbl_insert AFTER
    INSERT ON rg_kompetenzfestellung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_kompetenzfestellung_tbl_update AFTER
    UPDATE ON rg_kompetenzfestellung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_kompetenzfestellung_tbl_delete AFTER
    DELETE ON rg_kompetenzfestellung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- rg_angebote_bildenden_bereich_tbl
CREATE TRIGGER t1_rg_angebote_bildenden_bereich_tbl_insert AFTER
    INSERT ON rg_angebote_bildenden_bereich_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_bildenden_bereich_tbl_update AFTER
    UPDATE ON rg_angebote_bildenden_bereich_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_bildenden_bereich_tbl_delete AFTER
    DELETE ON rg_angebote_bildenden_bereich_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- rg_angebote_berufsvorbereitung_tbl
CREATE TRIGGER t1_rg_angebote_berufsvorbereitung_tbl_insert AFTER
    INSERT ON rg_angebote_berufsvorbereitung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_berufsvorbereitung_tbl_update AFTER
    UPDATE ON rg_angebote_berufsvorbereitung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_berufsvorbereitung_tbl_delete AFTER
    DELETE ON rg_angebote_berufsvorbereitung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- rg_angebote_berufliche_qualifizierung_tbl
CREATE TRIGGER t1_rg_angebote_berufliche_qualifizierung_tbl_insert AFTER
    INSERT ON rg_angebote_berufliche_qualifizierung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_berufliche_qualifizierung_tbl_update AFTER
    UPDATE ON rg_angebote_berufliche_qualifizierung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_berufliche_qualifizierung_tbl_delete AFTER
    DELETE ON rg_angebote_berufliche_qualifizierung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- rg_angebote_lebensbewaeltigung_tbl
CREATE TRIGGER t1_rg_angebote_lebensbewaeltigung_tbl_insert AFTER
    INSERT ON rg_angebote_lebensbewaeltigung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_lebensbewaeltigung_tbl_update AFTER
    UPDATE ON rg_angebote_lebensbewaeltigung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_rg_angebote_lebensbewaeltigung_tbl_delete AFTER
    DELETE ON rg_angebote_lebensbewaeltigung_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- ka_dokumente_tbl
CREATE TRIGGER t1_ka_dokumente_tbl_insert AFTER
    INSERT ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_dokumente_tbl_update AFTER
    UPDATE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_dokumente_tbl_delete AFTER
    DELETE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- nm_benutzer_master_tbl
CREATE TRIGGER t1_nm_benutzer_master_tbl_insert AFTER
    INSERT ON nm_benutzer_master_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_nm_benutzer_master_tbl_update AFTER
    UPDATE ON nm_benutzer_master_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_nm_benutzer_master_tbl_delete AFTER
    DELETE ON nm_benutzer_master_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- ka_status_tbl
CREATE TRIGGER t1_ka_status_tbl_update AFTER
    UPDATE ON ka_status_tbl
    FOR EACH ROW EXECUTE PROCEDURE status_lastaccess_log();

COMMIT;
